#include "microlib.h"

#define	GETFROMBROWSE_KEY		F3

/*****************************************************************************/
/* Function: int gkword_help(GET *get_ptr, char *data_buffer, int file_ref)     */
/* Description: This function is not called directly. It is used as a        */
/*              parameter to g4call(): g4call( gkword_menu, file_ref);          */
/*              Actually gkword_help() calls gkword() if F3 is pressed.            */
/* Parameters: GET *get_ptr        -> The get structure of the field.        */
/*             char *data_buffer   -> The currently entered information.     */
/*             int file_ref        -> The file reference number used for the */
/*                                    menu entries.                          */
/* Returns: A value significant for g4call routine.                          */
/*          >0  input data(as if it was read from the keyboard).             */
/*          =0  Read the next keyboard character as normal input data.       */
/*          -1  Call this call routine again without getting any keyboard    */
/*              data from the user.                                          */
/* Uses: gkword().                                                              */
/*****************************************************************************/
gkword_help( get_ptr, data_buffer, file_ref )
GET *get_ptr;
char *data_buffer;
int file_ref;
{
	D4DATA *dbfile;				/* Database file used for the menu. */
	int desc_len;
	int key_val ;					/* Key Pressed. */
	int count;						/* Counter. */
	int already_openf;			/* Already open file flag. */

	static changed_flag = FALSE;
	desc_len = get_ptr->width_data;


	key_val =  g4char() ;

	/* If F3 is pressed call gkword(). If gkword() returns Esc call the routine again.*/
	if ( key_val == GETFROMBROWSE_KEY ) {
		/* Get file pointer and set some frequently used variables. */
		dbfile = d4data( &set, dbf[file_ref]);
		if( dbfile == 0 ) {
			dbfile = d4open( &set, files(file_ref));
			already_openf = FALSE; /* I shall close the file later. */
			}
		else
			already_openf = TRUE;
		key_val =  gkword( get_ptr, data_buffer, file_ref ) ;
		if( already_openf == 0 ) {
			d4close(dbfile);
			}
		if ( key_val == ESC )  key_val = -1 ;
		return( key_val ) ;
		}

	/* If a key, which causes g4read to change field, is pressed, if the value */
	/* entered is found in the data base print the description. Otherwise ring */
	/* the bell and stay to the field.                                         */
	else if(key_val == RETURN || key_val == TAB || key_val == DOWN || key_val == UP \
				|| key_val == SHIFT_TAB) {
		/* If the data_buffer is blank...			*/
		if(!memcmp(data_buffer, space(desc_len),desc_len)) {
			if(! changed_flag)
				return(key_val);
			}
		changed_flag = FALSE;
		/* Get file pointer and set some frequently used variables. */
		dbfile = d4data( &set, dbf[file_ref]);
		if( dbfile == 0 ) {
			dbfile = d4open( &set, files(file_ref));
			already_openf = FALSE; /* I shall close the file later. */
			}
		else
			already_openf = TRUE;

		desc_len = get_ptr->width_data;

		/* If the data_buffer is blank...			*/
		if(!memcmp(data_buffer, space(desc_len), desc_len)) {
			w4num_att(get_ptr->row, get_ptr->col, space(desc_len), desc_len, custom[MAIN_I]);
			if(already_openf == FALSE)
				d4close(dbfile);
			return(key_val);
			}

		for(count=0; count<= desc_len -1; count ++)
			data_buffer[count] = gtoupper(data_buffer[count]);

		if( d4seek(dbfile, data_buffer) != 0 ) {
			d4unlock_index(dbfile);
			w4num_att(get_ptr->row, get_ptr->col, \
				space(desc_len), desc_len, custom[MAIN_I]);
			w4display("",mes[ME+4],mes[MH+16], (char *) 0);
			if(already_openf == FALSE)
				d4close(dbfile);
			return(HOME);
			}
		d4unlock_index(dbfile);
		w4num_att(get_ptr->row, get_ptr->col,\
				f4str(d4field_j(dbfile, 1)), desc_len, custom[MAIN_I]);
		if(already_openf == FALSE)
			d4close(dbfile);
		return(key_val);
		}
	else {
		changed_flag = TRUE;
		return(key_val);
		}
	}

/*****************************************************************************/
/* Function: int gkword(GET *get_ptr, char *data_buffer, int file_ref)          */
/* Description: This function is not called directly. It is used as a        */
/*              parameter to g4call(): g4call( gkword, file_ref);               */
/* Parameters: GET *get_ptr        -> The get structure of the field.        */
/*             char *data_buffer   -> The currently entered information.     */
/*             int file_ref        -> The file reference number used for the */
/*                                    menu entries.                          */
/* Returns: A value significant for g4call routine.                          */
/*          >0  input data(as if it was read from the keyboard).             */
/*          =0  Read the next keyboard character as normal input data.       */
/*          -1  Call this call routine again without getting any keyboard    */
/*              data from the user.                                          */
/* Uses: Nothing.                                                            */
/*****************************************************************************/
int gkword(get_ptr, data_buffer, file_ref)
GET *get_ptr;
char *data_buffer;
int file_ref;
{
	D4DATA *dbfile;				/* Database file used for the menu. */
	int browse_wref;        	/* Windows References. */
	int desc_len;
	int maxrow;						/* maxrow -1 = Maximum number of items in menu. */
	int cur_row;
	int key_val;					/* Key pressed. */
	int count;
	int skp;
	int seek_val;
	long cur_record;
	int skp_flag;

	/* Used in append and edit. */
	F4FIELD *f_desc;

							/* Code and Description Entry areas. Memory */
	char *desc;						/* is allocated before use.                 */
	char *sk;					/* Temporary search key. */

	maxrow = 5;
	skp = 0;
	skp_flag = 0;

	/* Get file pointer and set some frequently used variables. */
	dbfile = d4data( &set, dbf[file_ref]);
	d4tag_select(dbfile, d4tag(dbfile, "KEYWORD_T"));
	desc_len = get_ptr->width_data;

	/* Allocate space for CODE, DESC entry areas and TMP_KEY . */
	if( (desc = calloc(desc_len, sizeof(char))) == NULL) {
		w4display(mes[MT], mes[ME+1], (char *) 0);
		w4exit(1);
		}
	if( (sk = calloc(desc_len + 1, sizeof(char))) == NULL) {
		w4display(mes[MT], mes[ME+1], (char *) 0);
		w4exit(1);
		}
	memcpy(desc, space(desc_len), desc_len);
	memcpy(sk, space(desc_len), desc_len);

	desc[desc_len] = '\0';					/* Null terminate for seek operations. */

	for(count=0; count<= desc_len -1; count ++)
		data_buffer[count] = gtoupper(data_buffer[count]);

	if( d4seek(dbfile, data_buffer) != 0) {
		d4unlock_index(dbfile);
		/* If not in append mode ...  */
		if( memcmp(data_buffer, space(desc_len), desc_len))
			w4display("",mes[ME+4],mes[MH+16], (char *) 0);
		d4top(dbfile);
		d4unlock_index(dbfile);
		}
	d4unlock_index(dbfile);

	/* Open the browse window. 							*/

	browse_wref = w4define(12, 20, 12 + maxrow +2, 42);
	w4attribute(custom[BROWSER_N]);
	w4border(SINGLE, custom[BROWSER_B]);
	w4title(0, 1, mes[MT+16], custom[BROWSER_B]);
	w4popup();
	w4memory();
	w4activate( browse_wref );
	w4cursor(-1, -1);

	gkword_fillwindow(dbfile, desc_len, maxrow, desc_len);
	cur_row = 0;

	for (;;) {					/* ==L1== */
		key_val = g4char();

		if ( key_val == F1 ) {
			help("GET_FROM_KW");
			continue ;
			}
		/* ==If Alphanumeric== */
		if((key_val == BACK_SPACE) || \
			(key_val >= 32 && key_val <= 175) || \
			(key_val >= 224 && key_val <=233)) {

			if(key_val == BACK_SPACE) {
				if(skp == 0) {
					g4bell_set(1);
					g4bell();
					g4bell_set(0);
					skp_flag = 1;
					}
				else if(skp > 1) {
					skp --;
					skp --;
					}
				else if(skp == 1) {
					skp = 0;
					skp_flag = 1;
					d4top(dbfile);
					gkword_fillwindow(dbfile, desc_len, maxrow, desc_len);
					cur_row = 0;
					}
				}
			else if(skp == desc_len){
				g4bell_set(1);
				g4bell();
				g4bell_set(0);
				skp --;
				}
			else {
				sk[skp] = key_val;
				}

			if( !skp_flag) { /* ==If normal mode== */
				sk[skp + 1] = '\0';
				for(count=0 ; count <= skp; count ++)
					sk[count] = gtoupper(sk[count]);
				cur_record = d4recno(dbfile);
				seek_val = d4seek(dbfile, sk);
				d4unlock_index(dbfile);

				/* ==If found or just after it.==*/
				if(seek_val == r4after || seek_val == 0 || seek_val == r4eof) {
					/* If NOT really found... */
					if( memcmp(sk, f4str( d4field_j(dbfile, 1)), skp + 1)) {
						g4bell_set(1);
						g4bell();
						g4bell_set(0);
						seek_val = d4go(dbfile, cur_record);
						if( seek_val != 0 ) { /* If the record no longer exists. */
							d4top(dbfile);
							gkword_fillwindow(dbfile, desc_len, maxrow, desc_len);
							cur_row = 0;
							} /* If the record no longer exists. */
						} /* If NOT really found. */
					else { /* If really found... */
						/* Fill the window with records. (normal attribute). 					*/
						gkword_fillwindow(dbfile, desc_len, maxrow, skp + 1);
						cur_row = 0;
						skp ++;
						} /* If really found... */
					} /* ==If found or just after it.==*/
				} /* If normal mode */
			skp_flag = 0;
			} /* ==If Alphanumeric== */

		if(key_val == RETURN) {
			free(desc);
			free(sk);
			w4deactivate(browse_wref);
			w4close(browse_wref);
			/* Fill the buffer with the correct code and print the description */
			/* Next to the field.                                              */
			memcpy(data_buffer, f4str( d4field_j(dbfile, 1)), desc_len);
			d4seek(dbfile, data_buffer);
			d4unlock_index(dbfile);
			w4num_att(get_ptr->row, get_ptr->col , \
							f4str(d4field_j(dbfile, 1)), desc_len, custom[MAIN_I]);
			return(TAB);
			}
		if(key_val == ESC) {
			break;
			}
		if(key_val == DOWN) {
			skp = 0;
			/* Display with normal attribute the current row. */
			w4num_att(cur_row, 0, f4str( d4field_j(dbfile, 1)), desc_len, custom[BROWSER_N]);

			if (d4skip(dbfile, 1) == r4eof) {
				d4bottom(dbfile);
				cur_row --;
				}
			d4unlock_index(dbfile);
			if (cur_row == maxrow) {
				cur_row --;
				w4scroll(1);
				}
			cur_row ++;
			/* Display the bar */
			w4num_att(cur_row, 0, f4str( d4field_j(dbfile, 1)), desc_len, custom[BROWSER_S]);
			}

		else if (key_val == UP) {
			skp = 0;
			/* Display with normal attribute the current row. */
			w4num_att(cur_row, 0, f4str( d4field_j(dbfile, 1)), desc_len, custom[BROWSER_N]);

			if (d4skip(dbfile, -1)) {
				d4top(dbfile);
				cur_row ++;
				}
			d4unlock_index(dbfile);

			if (cur_row == 0) {
				cur_row ++;
				w4scroll(-1);
				}
			cur_row --;
			w4num_att(cur_row, 0, f4str( d4field_j(dbfile, 1)), desc_len, custom[BROWSER_S]);
			}
		}	/* ==L1== */

	/* Close the window and send to g4read() the key to execute. */
	free(desc);
	free(sk);
	w4deactivate(browse_wref);
	w4close(browse_wref);
	return(HOME);
	}

/*******************************************************************/
static gkword_fillwindow(dbfile, desc_len, maxrow, to_skp)
D4DATA *dbfile;
int	desc_len, to_skp;
int   maxrow;
{
	int count;
	int cur_row;

	w4clear(0);
	for(cur_row = 0; cur_row <= maxrow ; cur_row ++) {
		w4num_att(cur_row, 0, f4str( d4field_j(dbfile, 1)), desc_len, custom[BROWSER_N]);
		if( d4skip(dbfile, 1))
			break;
		d4unlock_index(dbfile);
		}
	d4unlock_index(dbfile);
	d4unlock_all_data(dbfile);
	if(cur_row > maxrow)
		cur_row = maxrow;
	/* Skip to the first record found. */
	for(count = 0 ; count <= cur_row; count ++) {
		d4skip(dbfile, -1);
		d4unlock_index(dbfile);
		}
	cur_row = 0;
	/* Display the bar */
	w4num_att(cur_row, 0, f4str( d4field_j(dbfile, 1)), to_skp, custom[BROWSER_S]);
	}
