#include "microlib.h"

#include "compat.h"
#include "ibmpc.h"
#include "dbfiles.h"
#include "kbd.h"

#define _ACTIVE     1
#define _INACTIVE   0
#define MAX_LISTBOX_ITEMS   20

long ListBox(RECT *, int, int, D4DATA *);
void AddListBoxItems(RECT *, char **);
int  ShowBar(RECT *, int, char *, int);
long TrackListBox(RECT *, int, char **);
int  poswin(RECT *);

extern VIDEOSTATUS vs;
static char *items[MAX_LISTBOX_ITEMS + 1];

long ListBox(RECT *rc, int nFileNum, int nFldNum, D4DATA *dbf)
{
    int    wref;
    char   *cr;
    char   szBuff[80];
    int    i = 0;
    int    size = 0;
    int    wide = rc->right - rc->left - 1;
    long   ret;

    wref = w4define(rc->top, rc->left, rc->bottom, rc->right);
    w4popup();
    w4memory();
    w4border(SINGLE, rc->border);
    w4title(0, 1, "TITLE", rc->title);
    w4attribute(rc->normal);
    w4activate(wref);
    w4clear(0);

    //do {
    //    size += idd->Field[i].nFSize;
    //} while (idd->Field[i++].szName);

    if ((cr = (char *) malloc(70)) == NULL ) {
        printf("Memory Error 1 !!!");
        w4exit(1);
    }

    for (i = 0; i <= MAX_LISTBOX_ITEMS; i++) {
        if ((items[i] = (char *) calloc(1, wide)) == NULL) {
            printf("Memory Error 2 !!!");
            w4exit(1);
        }
    }

    while (TRUE) {
        for (i = 0; i < MAX_LISTBOX_ITEMS; i++) {
            cr = (char *) d4record(dbf);
            cr += 8;
            memcpy((char *)&szBuff, cr, wide);
            szBuff[wide] = '\0';
            strcpy(items[i], (char *)&szBuff);
            if (d4skip(dbf, 0x01L) == r4eof)
                break;
        }

        // ensure a NULL terminator
        items[i+1] = NULL;

        AddListBoxItems(rc, items);
        ret = TrackListBox(rc, wref, items);

        if (HIWORD(ret) > 0)
            break;

        switch (LOWORD(ret)) {
            case VK_HOME :
                d4top(dbf);
                w4clear(0);
                break;
            case VK_DOWN :
                d4skip(dbf, -1L);
                w4clear(0);
                break;
            case VK_UP :
            default :
                break;
        }
    }

    for (i = 0; i <= MAX_LISTBOX_ITEMS; i++)
        free(items[i]);

    w4deactivate(wref);
    w4close(wref);

    return (ret);
}

void AddListBoxItems(RECT *rc, char **str)
{
    int x1, y1, x2, y2;
    char *c;

    switch (vs.crtmode) {
        case VMODE_3 :
        case VMODE_7 :
            x1 = rc->left + 1;
            y1 = rc->top + 1;
            x2 = rc->right;
            y2 = rc->bottom;
            while ((y2 - y1) && *str) {
                at_say(x1, y1, *str++, rc->normal);
                y1++;
            }
            break;
        //case VMODE_16 :
        //case VMODE_18 :
        //    x1 = (rc->left + 1) << 3;
        //    y1 = (rc->top + 1) << 4;
        //    x2 = rc->right << 3;
        //    y2 = rc->bottom << 4;
        //    while (*str && ((y2 - y1) << 4)) {
        //        _drawString(x1, y1, *str++, FG(w->wincolor), _CURRENT_FONT_);
        //        y1 += 16;
        //        x1 = (rc->left + 1) << 3;
        //    }
        //    break;
        default :
            break;
    }
}

int ShowBar(RECT *rc, int nPos, char *c, int flag)
{
    int x1, y1, x2, y2;
    unsigned char attr;

    if (flag)
        attr = rc->active;
    else
        attr = rc->normal;

    switch (vs.crtmode) {

        case VMODE_3 :
        case VMODE_7 :
            x1 = rc->left + 1;
            y1 = nPos;
            at_say(x1, y1, c, attr);
            break;

        default :
            break;

    }
}

long TrackListBox(RECT *rc, int window, char **pSels)
{
    unsigned int nCurrKey;
    unsigned int nItems = 0;
    unsigned int nListCurrent = 0;
    unsigned int nListSize = rc->bottom - rc->top - 2;
    unsigned int nCurrPos = rc->top + 1;
    char **sel = pSels;

    for (; *sel; sel++)
        if (nItems < MAX_LISTBOX_ITEMS)
            nItems++;

    if ((nItems - 1) < nListSize)
        nListSize = nItems;

    sel = pSels; // restore pointer

    ShowBar(rc, nCurrPos, pSels[0], _ACTIVE);

    while (1) {
        nCurrKey = _kbdhit();
        //tptr = memScanEQ(m->mkeys, select, nCurrKeys);
        //if (tptr != 0xff)
        //    break;

        switch (nCurrKey) {

            case VK_F9 : {
                char WinInfo[4096];
                poswin(rc);
                w4read_window(window, (char *)&WinInfo);
                w4deactivate(window);
                w4close(window);
                window = w4define(rc->top, rc->left, rc->bottom, rc->right);
                w4popup();
                w4memory();
                w4border(SINGLE, rc->border);
                w4title(0, 1, "TITLE", rc->title);
                w4attribute(rc->normal);
                w4write_window(window, (char *)&WinInfo);
                w4activate(window);
                AddListBoxItems(rc, sel);
                ShowBar(rc, nCurrPos, pSels[nListCurrent], _ACTIVE);
                break;
                }

            case VK_ESCAPE :
                break;

            case VK_ENTER :
                break;

            case VK_DOWN :
                if (nListCurrent + 1 == nItems) {
                    putchar(0x07);
                    return MAKELONG(nCurrKey, 0);
                }
                ShowBar(rc, nCurrPos, pSels[nListCurrent], _INACTIVE);
                nListCurrent++;
                if (nListCurrent > nListSize) {
                    sel++;
                    AddListBoxItems(rc, sel);
                }
                else
                    nCurrPos++;
                ShowBar(rc, nCurrPos, pSels[nListCurrent], _ACTIVE);
                break;

            case VK_UP :
                if (nListCurrent == 0) {
                    putchar(0x07);
                    return MAKELONG(nCurrKey, 0);
                }
                ShowBar(rc, nCurrPos, pSels[nListCurrent], _INACTIVE);
                nListCurrent--;
                if (nCurrPos < rc->top + 2) {
                    sel--;
                    AddListBoxItems(rc, sel);
                }
                else
                    nCurrPos--;
                ShowBar(rc, nCurrPos, pSels[nListCurrent], _ACTIVE);
                break;

            case VK_HOME :
                putchar(0x07);
                return MAKELONG(nCurrKey, 0);
                //ShowBar(rc, nCurrPos, pSels[nListCurrent], _INACTIVE);
                //nListCurrent = 0;
                //nCurrPos = rc->top + 1;
                //sel = pSels;
                //AddListBoxItems(rc, sel);
                //ShowBar(rc, nCurrPos, pSels[nListCurrent], _ACTIVE);
                //break;

        }
        if (nCurrKey == VK_ESCAPE || nCurrKey == VK_ENTER)
            break;
    }
    nListCurrent++;

    return MAKELONG(nCurrKey, nCurrKey == VK_ESCAPE ? -1 : nListCurrent);
}

int poswin(RECT *rc)
{
    int  pop_window = 0;
    RECT *rrc = rc;
    BOOL bDone = FALSE;
    int  nCurrKey;

    pop_window = w4define( rrc->top, rrc->left, rrc->bottom, rrc->right ) ;
    w4attribute( F_INTENSE | F_BLUE | F_GREEN ) ;
    w4popup() ;
    w4memory() ;
    w4border( PANEL, F_GREEN ) ;
    w4activate( pop_window ) ;

    while (!bDone) {
        nCurrKey = _kbdhit();

        switch ( nCurrKey ) {
            case VK_ENTER :
                bDone = TRUE;
                rc->top    = rrc->top;
                rc->left   = rrc->left;
                rc->bottom = rrc->bottom;
                rc->right  = rrc->right;

                w4deactivate( pop_window ) ;
                w4close( pop_window ) ;
                return (1);

            case VK_UP :
                w4deactivate(pop_window);
                w4close(pop_window);
                rrc->top-- ;
                if ( rrc->top >= 0 )
                    rrc->bottom-- ;
                break ;

            case VK_DOWN :
                w4deactivate(pop_window);
                w4close(pop_window);
                rrc->bottom++ ;
                if ( rrc->bottom <= 23 )
                    rrc->top++ ;
                break ;

            case VK_RIGHT :   /* Move Right */
                w4deactivate(pop_window);
                w4close(pop_window);
                rrc->right++;
                if ( rrc->right <= 79 )
                    rrc->left++ ;
                break ;

            case VK_LEFT :   /* Move Left */
                w4deactivate(pop_window);
                w4close(pop_window);
                rrc->left-- ;
                if ( rrc->left >= 0 )
                    rrc->right-- ;
                break ;

            default :
                continue;
        }

        pop_window = w4define( rrc->top, rrc->left, rrc->bottom, rrc->right ) ;
        w4attribute( F_INTENSE | F_BLUE | F_GREEN ) ;
        w4popup() ;
        w4border( PANEL, F_GREEN ) ;
        w4activate(pop_window);
    }

}   
